// Copyright (C) 2005 Alexander Ushakov
// Contents: Definition of class ShftConjKeyInstance
//
// Principal Authors: Alexander Ushakov
//
// Revision History:
//

#ifndef _ShftConjKeyGeneration_h_
#define _ShftConjKeyGeneration_h_

#include "Word.h"

#include <vector>
using namespace std;


//---------------------------------------------------------------------------//
//-------------------------- ShftConjKeyInstance ----------------------------//
//---------------------------------------------------------------------------//


class ShftConjKeyInstance
{
  
  /////////////////////////////////////////////////////////
  //                                                     //
  //  Constructors:                                      //
  //                                                     //
  /////////////////////////////////////////////////////////
  
 public:

  //! Create an instance of the protocol.
  ShftConjKeyInstance( int braid_rank , Word base_word , Word keyA , Word KeyB );
  
  
  /////////////////////////////////////////////////////////
  //                                                     //
  //  Accessors:                                         //
  //                                                     //
  /////////////////////////////////////////////////////////

 public:
  
  //! Generate a random instance of the protocol
  /*!
    \param braid_rank - rank of the braid group;
    \param baseLenth - the length of the base word
    \param keyLength - the length of the key
   */
  static ShftConjKeyInstance random( int braid_rank , int baseLenth , int keyLength );

  //! (accessor function) get the rank of the braid group
  int  getBraidRank  ( ) const { return theRank       ; }
  //! (accessor function) get the private key of the first  party
  Word getPrivateKeyA( ) const { return thePrivateKeyA; }
  //! (accessor function) get the private key of the second party
  Word getPrivateKeyB( ) const { return thePrivateKeyB; }
  //! (accessor function) get the public  key of the first  party
  Word getPublicKeyA ( ) const { return thePublicKeyA ; }
  //! (accessor function) get the public  key of the second party
  Word getPublicKeyB ( ) const { return thePublicKeyB ; }
  
  /////////////////////////////////////////////////////////
  //                                                     //
  //  Data members:                                      //
  //                                                     //
  /////////////////////////////////////////////////////////

 private:

  //! the rank of the braid group
  int theRank;

  //! the base element (denoted by \f$w\f$)
  Word theBase;

  //! the private key of the first party (denoted by \f$A\f$)
  Word thePrivateKeyA;

  //! the public key of the first party \f$P_B = D(sh(A)^{-1} \sigma_1 sh(w) A)\f$
  Word thePublicKeyA;

  //! the private key of the second party (denoted by \f$B\f$)
  Word thePrivateKeyB;

  //! the public key of the second party \f$P_B = D(sh(B)^{-1} \sigma_1 w B)\f$
  Word thePublicKeyB;

};


//---------------------------------------------------------------------------//
//------------------------------- Algorithms --------------------------------//
//---------------------------------------------------------------------------//


Word generatorShift( const Word& w );


#endif
