// Copyright (C) 2005 Alexander Ushakov
// Contents: Definition of class Equation
//
// Principal Authors: Alexander Ushakov
//
// Revision History:
//


#ifndef _Equation_h__
#define _Equation_h__


#include "GraphType.h"
#include "GraphConcept.h"
#include "GraphConceptAlgorithms.h"
using namespace Graphs;

#include "Word.h"


//---------------------------------------------------------------------------//
//--------------------------------- Equation --------------------------------//
//---------------------------------------------------------------------------//


class Equation
{

  /////////////////////////////////////////////////////////
  //                                                     //
  //  Constructors:                                      //
  //                                                     //
  /////////////////////////////////////////////////////////
  
 public:

  Equation( int nGen , int nVar , const Word& eq );

  /////////////////////////////////////////////////////////
  //                                                     //
  //  Accessors:                                         //
  //                                                     //
  /////////////////////////////////////////////////////////

 public:


  //! Get the number of the generators of the group
  int getTheNumberOfGenerators( ) const { return theNumberOfGenerators; }
  //! Get the number of variables in the formula
  int getTheNumberOfVariables( ) const { return theNumberOfVariables; }
  //! Get the word presentation of the equation
  const Word& getTheEquation( ) const { return theEquation; }
  
  //! Determine if the equation is quadratic
  bool isQuadratic( ) const;
  
  //! Construct the transformation graph for the equation (finite for quadratic equation)
  pair< IntLabeledGraph , map< Word , int > > constructTransformationGraph( ) const;
  
  //! Determine if the equation has trivial solution
  bool trivialSolution( ) const;
  
  /////////////////////////////////////////////////////////
  //                                                     //
  //  I/O:                                               //
  //                                                     //
  /////////////////////////////////////////////////////////

 private:
  
  friend ostream& operator << ( ostream& os , const Equation& eq );
  
  /////////////////////////////////////////////////////////
  //                                                     //
  //  Internal functions:                                //
  //                                                     //
  /////////////////////////////////////////////////////////

 private:
  
  bool isGenerator( int g ) const;
  bool isVariable ( int g ) const;
  
  //! (Auxiliary) Used in constructTransformationGraph(...)
  void addEquationToTransformationGraph( const Word& eq , set< Word >& chk_eq , set< Word >& new_eq , const set< int >& permutableGenerators , int x , const Word& im ) const;
  
  
  /////////////////////////////////////////////////////////
  //                                                     //
  //  Data members:                                      //
  //                                                     //
  /////////////////////////////////////////////////////////

 private:

  //! the number of the generators of the group
  int theNumberOfGenerators;
  //! the number of variables in the formula (the actual number of variables in the equation can be lesser)
  int theNumberOfVariables;
  //! the presentation of the equation
  /*!
    A word theEquation is a sequence of generators. Each generator \f$g\f$ is interpreted the following way:
    - if \f$|g| \le theNumberOfGenerators\f$ then \f$g\f$ is the corresponding generator of the group
    - if \f$|g| > theNumberOfGenerators\f$ then \f$g\f$ is a variable with index \f$|g| - theNumberOfGenerators\f$
    raised in the power \f$\pm 1\f$ depending on the sign of \f$g\f$.
   */
  Word theEquation;

};


#endif

