// Copyright (C) 2005 Alexander Ushakov
// Contents: Definition of class SubgroupFG
//
// Principal Authors: Alexander Ushakov
//
// Revision History:
//

#ifndef _SubgroupFG_h_
#define _SubgroupFG_h_


#include "Word.h"
#include "GraphType.h"
#include "GraphConcept.h"
#include "GraphConceptAlgorithms.h"
using namespace Graphs;


#include "vector"
using namespace std;


//---------------------------------------------------------------------------//
//------------------------------ SubgroupFG ---------------------------------//
//---------------------------------------------------------------------------//


class SubgroupFG
{
  /////////////////////////////////////////////////////////
  //                                                     //
  //  Constructors                                       //
  //                                                     //
  /////////////////////////////////////////////////////////
public:
  SubgroupFG( );
  SubgroupFG( int n_gens , const vector< Word >& gens );
  
protected:
  SubgroupFG( int n_gens , const IntLabeledGraph& fsa );


  /////////////////////////////////////////////////////////
  //                                                     //
  //  Operators:                                         //
  //                                                     //
  /////////////////////////////////////////////////////////
public:
  bool operator== ( const SubgroupFG& S ) const;
  SubgroupFG operator* ( const SubgroupFG& S ) const;
  
  /////////////////////////////////////////////////////////
  //                                                     //
  //  Accessors:                                         //
  //                                                     //
  /////////////////////////////////////////////////////////
public:

  const vector< Word >& getGenerators( ) const { return theGenerators; }
  const vector< Word >& getNielsenGenerators( ) const;
  
  const IntLabeledGraph& getFSA( ) const;
  bool doesBelong( const Word& w ) const;
  
  int getIndex( ) const;
  int getRank( ) const;
  
  Word express( const Word& w ) const;
  
  /////////////////////////////////////////////////////////
  //                                                     //
  //  Internal functions:                                //
  //                                                     //
  /////////////////////////////////////////////////////////
  
private:


  void computeFSA( ) const;
  void computeNielsenGenerators( ) const;
  
  /////////////////////////////////////////////////////////
  //                                                     //
  //  Data members                                       //
  //                                                     //
  /////////////////////////////////////////////////////////
  
private:

  vector< Word > theGenerators;
  int theNumberOfGenerators;
  
  mutable bool fsaDone;
  mutable IntLabeledGraph theFSA;
  mutable list< FoldDetails< IntLabeledGraph::vertex_type , IntLabeledGraph::edge_type > > foldDetails;
  
  mutable bool nielsDone;
  mutable vector< Word > theNielsenGenerators;
};


//---------------------------------------------------------------------------//
//------------------------------ Substitute ---------------------------------//
//---------------------------------------------------------------------------//


template < class ConstIterator >
Word substitute( ConstIterator B , ConstIterator E , const vector< Word >& wrds ) 
{
  Word result;
  for( ; B!=E ; ++B ) {
    int letter = *B;
    if( letter>0 )
      result *=  wrds[ letter-1];
    else
      result *= -wrds[-letter-1];
  }
  
  return result;
}


Word substitute( const Word& w , const vector< Word >& wrds );


//---------------------------------------------------------------------------//
//------------------------------ operator << --------------------------------//
//---------------------------------------------------------------------------//

ostream& operator << ( ostream& os , const SubgroupFG& sbgp );



#endif

